#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>
#include "1wire.h"
#include "lcd.h"
#include "keyb.h"
#include "i2c.h"
#include "MCP79410.h"
#include "eeprom.h"
#include "usart.h"

#define DS18B20_CONVERT_T 0x44
#define DS18B20_READ_SCRATCHPAD 0xBE

typedef struct {
	DateTime dateTime;
	float temperature;
} TemperatureRecord;

enum TemperatureState {
	TEMP_IDLE,
	TEMP_IN_PROGRESS,
	TEMP_READY
};

void getTemperature(float * temperature);
void setDateTime(void);
void displayDateTime(void);
void browseRecords(uint16_t currentRecordAddress);
void displayRecord(TemperatureRecord * temperatureRecord);
void getSeparatedNumbers(char str[], uint16_t outArray[], uint8_t outArraySize);
void sendRecords(void);

char buffer[64];
volatile uint8_t bufferIndex = 0;
volatile uint8_t newCommand = 0;

int main(void) {
	i2cInit();
	keybInit();
	usartInit();
	lcdInit();
	lcdInitPrintf();
	sei();
	//odczyt adresu ostatnio zapisanego rekordu w EEPROM
	uint16_t lastRecordAddress;
	eepromReadBytesInternal(0, (uint8_t*) &lastRecordAddress, sizeof(lastRecordAddress));
	if (lastRecordAddress > 1023 - sizeof(TemperatureRecord)) lastRecordAddress = 2;
	uint16_t browsingRecordAddress = lastRecordAddress;
	uint8_t recordWritten = 0;
	TemperatureRecord temperatureRecord;
	uint8_t browsing = 0;
	while(1){
		//odczyt temperatury
		getTemperature(&temperatureRecord.temperature);
		//odczyt daty i czasu
		DateTime * dateTime = &temperatureRecord.dateTime;
		rtcReadDateTime(dateTime);
		//zapis rekordu
		if ((dateTime->seconds % 15) == 0) {
			if (!recordWritten) {
				lastRecordAddress += sizeof(temperatureRecord);
				if (lastRecordAddress > 1023 - sizeof(temperatureRecord)) lastRecordAddress = 2;
				browsingRecordAddress = lastRecordAddress;
				eepromWriteBytesInternal(lastRecordAddress, (uint8_t *) &temperatureRecord, sizeof(temperatureRecord));
				eepromWriteBytesInternal(0, (uint8_t*) &lastRecordAddress, sizeof(lastRecordAddress));
				recordWritten = 1;
			}
		} else {
			recordWritten = 0;
		}
		//wywietlenie danych
		if (browsing) {
			TemperatureRecord historicRecord;
			eepromReadBytesInternal(browsingRecordAddress, (uint8_t *) &historicRecord, sizeof(temperatureRecord));
			displayRecord(&historicRecord);
		} else {
			displayRecord(&temperatureRecord);
		}
		//obsuga klawiatury
		if (readKeyboard()) {
			uint8_t key = getKey();
			if (browsing) {
				if (key == 1) {
					browsingRecordAddress += sizeof(temperatureRecord);
				}
				if (key == 5) {
					browsingRecordAddress -= sizeof(temperatureRecord);
				}
				if (browsingRecordAddress > 1023 - sizeof(temperatureRecord)) browsingRecordAddress = 2;
				if (key == 16) browsing = 0;
			} else {
				if (key == 1) setDateTime();
				if (key == 2) browsing = 1;
			}
		}
		if (newCommand) {
			newCommand = 0;
			if (strncmp(buffer, "DATAGET", 7) == 0) {
				sendRecords();
			}
			if (strncmp(buffer, "DATETIMESET", 11) == 0) {
				DateTime newDateTime;
				uint16_t datetime[6];
				getSeparatedNumbers(buffer + 12, datetime, 6);
				newDateTime.hours = datetime[0];
				newDateTime.minutes = datetime[1];
				newDateTime.seconds = datetime[2];
				newDateTime.year = datetime[3];
				newDateTime.month = datetime[4];
				newDateTime.dayOfMonth = datetime[5];
				rtcWriteDateTime(&newDateTime);
			}
		}
	}
}

void getSeparatedNumbers(char str[], uint16_t outArray[], uint8_t outArraySize) {
	uint8_t strPos = 0;
	uint8_t outPos = 0;
	for (uint8_t i = 0; i < outArraySize; i++) outArray[i] = 0;
	while ((str[strPos] != 0) && (outPos < outArraySize)) {
		if (str[strPos] < '0' || str[strPos] > '9') {
			outPos++;
		} else {
			outArray[outPos] = outArray[outPos] * 10 + (str[strPos] - '0');
		}
		strPos++;
	}
}

void displayRecord(TemperatureRecord * temperatureRecord) {
	DateTime * dateTime = &temperatureRecord->dateTime;
	lcdGotoXY(0, 0);
	printf("%02d:%02d:%02d", dateTime->hours, dateTime->minutes, dateTime->seconds);
	lcdGotoXY(9, 0);
	printf("t=%.1f\xDF", temperatureRecord->temperature);
	lcdGotoXY(0, 1);
	printf("%02d-%02d-%4d", dateTime->dayOfMonth, dateTime->month, dateTime->year);
}

void sendRecord(TemperatureRecord * temperatureRecord) {
	char str[64];
	DateTime * dateTime = &temperatureRecord->dateTime;
	uint8_t pos = sprintf(str, "%02d:%02d:%02d ", dateTime->hours, dateTime->minutes, dateTime->seconds);
	pos += sprintf(str + pos, "%02d-%02d-%4d ", dateTime->dayOfMonth, dateTime->month, dateTime->year);
	sprintf(str + pos, "t=%.1f\r\n", temperatureRecord->temperature);
	usartSendString(str);
}

void sendRecords(void) {
	uint16_t recordAddress = 2;
	while (recordAddress <= 1023 - sizeof(TemperatureRecord)) {
		TemperatureRecord historicRecord;
		eepromReadBytesInternal(recordAddress, (uint8_t *) &historicRecord, sizeof(historicRecord));
		sendRecord(&historicRecord);
		recordAddress += sizeof(TemperatureRecord);
	}
}

void getTemperature(float * temperature) {
	static enum TemperatureState temperatureState = TEMP_IDLE;
	switch (temperatureState) {
		case TEMP_IDLE:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_CONVERT_T);
			temperatureState = TEMP_IN_PROGRESS;
			break;
		case TEMP_IN_PROGRESS:
			if (oneWireReceiveBit()) temperatureState = TEMP_READY;
			break;
		case TEMP_READY:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_READ_SCRATCHPAD);
			int16_t temp;
			uint8_t * byte = (uint8_t *) &temp;
			byte[0] = oneWireReceiveByte();
			byte[1] = oneWireReceiveByte();
			*temperature = temp / 16.0;
			temperatureState = TEMP_IDLE;
			break;
	}
}

void setDateTime(void) {
	//wcz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_CURSOR | LCD_PARAM_ON_OFF_DISPLAY);
	//wczytaj czas
	lcdGotoXY(0, 0);
	printf("  :  :  ");
	lcdGotoXY(0, 0);
	const uint8_t timeFieldsSizes[] = {2, 2, 2};
	uint16_t time[3];
	readSeparatedNumbers(timeFieldsSizes, 3, time);
	//wczytaj dat
	lcdGotoXY(0, 1);
	printf("  -  -    ");
	lcdGotoXY(0, 1);
	const uint8_t dateFieldsSizes[] = {2, 2, 4};
	uint16_t date[3];
	readSeparatedNumbers(dateFieldsSizes, 3, date);
	DateTime newDateTime;
	newDateTime.hours = time[0];
	newDateTime.minutes = time[1];
	newDateTime.seconds = time[2];
	newDateTime.dayOfMonth = date[0];
	newDateTime.month = date[1];
	newDateTime.year = date[2];
	//zapisz nowy czas i dat
	rtcWriteDateTime(&newDateTime);
	//wycz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_DISPLAY);
}

ISR(USART_RXC_vect) {
	uint8_t value = UDR;
	if (value != '\r') {
		if (bufferIndex < sizeof(buffer)) {
			buffer[bufferIndex++] = value;
		}
		} else {
		buffer[bufferIndex] = 0;
		bufferIndex = 0;
		newCommand = 1;
	}
}